@@ -44,42 +44,23 @@
 #include "gedit2.h"
 #include "gedit-menus.h"
 #include "gedit-debug.h"
-#include "gedit-prefs-manager.h"
+#include "gedit-prefs.h"
 #include "gedit-recent.h" 
 #include "gedit-file.h"
 #include "gedit-view.h"
 #include "gedit-utils.h"
 #include "gedit-plugins-engine.h"
-#include "gedit-output-window.h"
-#include "recent-files/egg-recent-view-bonobo.h"
-#include "recent-files/egg-recent-view-gtk.h"
-#include "recent-files/egg-recent-model.h"
 
 #include <bonobo/bonobo-ui-util.h>
 #include <bonobo/bonobo-control.h>
 
 #include <gconf/gconf-client.h>
 
-#define RECENT_KEY 		"GeditRecent"
-#define OUTPUT_WINDOW_KEY	"GeditOutputWindow"
+#define RECENT_KEY "GeditRecent"
 
 struct _GeditMDIPrivate
 {
-	gint dummy;
-};
-
-typedef struct _GeditWindowPrefs	GeditWindowPrefs;
-
-struct _GeditWindowPrefs
-{
-	gboolean toolbar_visible;
-	GeditToolbarSetting toolbar_buttons_style;
-
-	gboolean statusbar_visible;
-	gboolean statusbar_show_cursor_position;
-	gboolean statusbar_show_overwrite_mode;
-
-	gboolean output_window_visible;
+	gint untitled_number;
 };
 
 static void gedit_mdi_class_init 	(GeditMDIClass	*klass);
@@ -105,8 +86,6 @@ static void gedit_mdi_child_state_changed_handler (GeditMDIChild *child);
 
 static void gedit_mdi_set_active_window_undo_redo_verbs_sensitivity (BonoboMDI *mdi);
 
-static void gedit_mdi_app_destroy_handler (BonoboMDI *mdi, BonoboWindow *window);
-
 static void gedit_mdi_view_menu_item_toggled_handler (
 			BonoboUIComponent           *ui_component,
 			const char                  *path,
@@ -114,30 +93,8 @@ static void gedit_mdi_view_menu_item_toggled_handler (
 			const char                  *state,
 			BonoboWindow                *win);
 
-
-static GQuark window_prefs_id = 0;
-
-static GeditWindowPrefs *gedit_window_prefs_new 		(void);
-static void		 gedit_window_prefs_attach_to_window 	(GeditWindowPrefs *prefs,
-								 BonoboWindow 	  *win);
-static GeditWindowPrefs	*gedit_window_prefs_get_from_window 	(BonoboWindow     *win);
-static void		 gedit_window_prefs_save 		(GeditWindowPrefs *prefs);
-
 static BonoboMDIClass *parent_class = NULL;
 
-enum
-{
-	TARGET_URI_LIST = 100
-};
-
-static GtkTargetEntry drag_types[] =
-{
-	{ "text/uri-list", 0, TARGET_URI_LIST },
-};
-
-static gint n_drag_types = sizeof (drag_types) / sizeof (drag_types [0]);
-
-
 GType
 gedit_mdi_get_type (void)
 {
@@ -177,139 +134,20 @@ gedit_mdi_class_init (GeditMDIClass *klass)
   	object_class->finalize = gedit_mdi_finalize;
 }
 
-static void
-menu_position_under_widget (GtkMenu *menu, int *x, int *y,
-			    gboolean *push_in, gpointer user_data)
-{
-	GtkWidget *w;
-	int width, height;
-	int screen_width, screen_height;
-	GtkRequisition requisition;
-
-	w = GTK_WIDGET (user_data);
-	
-	gdk_drawable_get_size (w->window, &width, &height);
-	gdk_window_get_origin (w->window, x, y);
-	*y = *y + height;
-
-	gtk_widget_size_request (GTK_WIDGET (menu), &requisition);
-
-	screen_width = gdk_screen_width ();
-	screen_height = gdk_screen_height ();
-
-	*x = CLAMP (*x, 0, MAX (0, screen_width - requisition.width));
-	*y = CLAMP (*y, 0, MAX (0, screen_height - requisition.height));
-}
-
-static gboolean
-open_button_pressed_cb (GtkWidget *widget,
-			      GdkEventButton *event,
-			      gpointer *user_data)
-{
-	GtkWidget *menu;
-	GeditMDI *mdi;
-
-	g_return_val_if_fail (GTK_IS_BUTTON (widget), FALSE);
-	g_return_val_if_fail (GEDIT_IS_MDI (user_data), FALSE);
-
-	mdi = GEDIT_MDI (user_data);
-
-	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (widget), TRUE);
-
-	menu = g_object_get_data (G_OBJECT (widget), "recent-menu");
-	gnome_popup_menu_do_popup_modal (menu,
-				menu_position_under_widget, widget,
-				event, widget, widget);
-	
-	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (widget), FALSE);
-	
-	return TRUE;
-}
-
-static gboolean
-open_button_key_pressed_cb (GtkWidget *widget,
-				  GdkEventKey *event,
-				  gpointer *user_data)
-{
-	if (event->keyval == GDK_space ||
-	    event->keyval == GDK_KP_Space ||
-	    event->keyval == GDK_Return ||
-	    event->keyval == GDK_KP_Enter) {
-		open_button_pressed_cb (widget, NULL, user_data);
-	}
-
-	return FALSE;
-}
-
-
-
-static void
-gedit_mdi_add_open_button (GeditMDI *mdi, BonoboUIComponent *ui_component,
-			 const gchar *path, const gchar *tooltip)
-{
-	GtkWidget *menu;
-	EggRecentViewGtk *view;
-	EggRecentModel *model;
-	BonoboUIToolbarItem *item;
-	BonoboControl *wrapper;
-	GtkWidget *button;
-
-	item = BONOBO_UI_TOOLBAR_ITEM (bonobo_ui_toolbar_item_new ());
-
-	button = gtk_toggle_button_new ();
-	gtk_button_set_relief (GTK_BUTTON (button), GTK_RELIEF_NONE);
-
-	gtk_container_add (GTK_CONTAINER (button),
-			   gtk_arrow_new (GTK_ARROW_DOWN, GTK_SHADOW_OUT));
-
-	gtk_container_add (GTK_CONTAINER (item), button);
-
-	gtk_widget_show_all (GTK_WIDGET (item));
-
-	model = gedit_recent_get_model ();
-
-	menu = gtk_menu_new ();
-	gtk_widget_show (menu);
-	view = egg_recent_view_gtk_new (menu, NULL);
-	g_signal_connect (view, "activate",
-			  G_CALLBACK (gedit_file_open_recent), NULL);
-	egg_recent_view_gtk_show_icons (view, TRUE);
-	egg_recent_view_gtk_show_numbers (view, FALSE);
-	egg_recent_view_set_model (EGG_RECENT_VIEW (view), model);
-	g_object_set_data (G_OBJECT (button), "recent-menu", menu);
-	
-	g_signal_connect_object (button, "key_press_event",
-				 G_CALLBACK (open_button_key_pressed_cb),
-				 mdi, 0);
-	g_signal_connect_object (button, "button_press_event",
-				 G_CALLBACK (open_button_pressed_cb),
-				 mdi, 0);
-
-	wrapper = bonobo_control_new (GTK_WIDGET (item));
-	bonobo_ui_component_object_set (ui_component,
-					path,
-					BONOBO_OBJREF (wrapper),
-					NULL);
-
-	bonobo_object_unref (wrapper);
-}
-
 static void 
 gedit_mdi_init (GeditMDI  *mdi)
 {
 	gedit_debug (DEBUG_MDI, "START");
 
-	bonobo_mdi_construct (BONOBO_MDI (mdi), 
-			      "gedit-2", 
-			      "gedit",
-			      gedit_prefs_manager_get_default_window_width (),
-			      gedit_prefs_manager_get_default_window_height ());
+	bonobo_mdi_construct (BONOBO_MDI (mdi), "gedit-2", "gedit");
 	
 	mdi->priv = g_new0 (GeditMDIPrivate, 1);
 
+	mdi->priv->untitled_number = 0;	
+
 	bonobo_mdi_set_ui_template_file (BONOBO_MDI (mdi), GEDIT_UI_DIR "gedit-ui.xml", gedit_verbs);
 	
-	bonobo_mdi_set_child_list_path (BONOBO_MDI (mdi), "/menu/Documents/OpenDocuments/");
+	bonobo_mdi_set_child_list_path (BONOBO_MDI (mdi), "/menu/Documents/");
 
 	/* Connect signals */
 	g_signal_connect (G_OBJECT (mdi), "top_window_created",
@@ -329,14 +167,12 @@ gedit_mdi_init (GeditMDI  *mdi)
 			  G_CALLBACK (gedit_mdi_child_changed_handler), NULL);
 	g_signal_connect (G_OBJECT (mdi), "view_changed",
 			  G_CALLBACK (gedit_mdi_view_changed_handler), NULL);
+
 	
 	g_signal_connect (G_OBJECT (mdi), "all_windows_destroyed",
 			  G_CALLBACK (gedit_file_exit), NULL);
-
-	g_signal_connect (G_OBJECT (mdi), "top_window_destroy",
-			  G_CALLBACK (gedit_mdi_app_destroy_handler), NULL);
-
 			  
+
 	gedit_debug (DEBUG_MDI, "END");
 }
 
@@ -380,18 +216,21 @@ gedit_mdi_new (void)
 	return mdi;
 }
 
-
 static void
 gedit_mdi_app_created_handler (BonoboMDI *mdi, BonoboWindow *win)
 {
 	GtkWidget *widget;
 	BonoboControl *control;
 	BonoboUIComponent *ui_component;
-	EggRecentView *view;
-	EggRecentModel *model;
-	GeditWindowPrefs *prefs;
-	GdkWindowState state;
-	
+	GeditRecent *recent;
+
+	static GtkTargetEntry drag_types[] =
+	{
+		{ "text/uri-list", 0, 0 },
+	};
+
+	static gint n_drag_types = sizeof (drag_types) / sizeof (drag_types [0]);
+
 	gedit_debug (DEBUG_MDI, "");
 	
 	ui_component = bonobo_mdi_get_ui_component_from_window (win);
@@ -440,23 +279,11 @@ gedit_mdi_app_created_handler (BonoboMDI *mdi, BonoboWindow *win)
 
 	g_object_set_data (G_OBJECT (win), "OverwriteMode", widget);
 
-	/* Add custom Open button to toolbar */
-	gedit_mdi_add_open_button (GEDIT_MDI (mdi),
-				   ui_component, "/Toolbar/FileOpenMenu",
-				   _("Open a file."));
-
-	prefs = gedit_window_prefs_new ();
-	gedit_window_prefs_attach_to_window (prefs, win);
-
 	/* Set the statusbar style according to prefs */
 	gedit_mdi_set_app_statusbar_style (win);
 	
 	/* Set the toolbar style according to prefs */
 	gedit_mdi_set_app_toolbar_style (win);
-
-	gedit_menus_set_verb_state (ui_component, 
-				    "/commands/ViewOutputWindow",
-				    prefs->output_window_visible);
 		
 	/* Add listener fo the view menu */
 	bonobo_ui_component_add_listener (ui_component, "ViewToolbar", 
@@ -465,9 +292,6 @@ gedit_mdi_app_created_handler (BonoboMDI *mdi, BonoboWindow *win)
 	bonobo_ui_component_add_listener (ui_component, "ViewStatusbar", 
 			(BonoboUIListenerFn)gedit_mdi_view_menu_item_toggled_handler, 
 			(gpointer)win);
-	bonobo_ui_component_add_listener (ui_component, "ViewOutputWindow", 
-			(BonoboUIListenerFn)gedit_mdi_view_menu_item_toggled_handler, 
-			(gpointer)win);
 
 	bonobo_ui_component_add_listener (ui_component, "ToolbarSystem", 
 			(BonoboUIListenerFn)gedit_mdi_view_menu_item_toggled_handler, 
@@ -478,7 +302,7 @@ gedit_mdi_app_created_handler (BonoboMDI *mdi, BonoboWindow *win)
 	bonobo_ui_component_add_listener (ui_component, "ToolbarIconText", 
 			(BonoboUIListenerFn)gedit_mdi_view_menu_item_toggled_handler, 
 			(gpointer)win);
-	bonobo_ui_component_add_listener (ui_component, "ToolbarIconBothHoriz", 
+	bonobo_ui_component_add_listener (ui_component, "ToolbarTooltips", 
 			(BonoboUIListenerFn)gedit_mdi_view_menu_item_toggled_handler, 
 			(gpointer)win);
 
@@ -488,64 +312,30 @@ gedit_mdi_app_created_handler (BonoboMDI *mdi, BonoboWindow *win)
 	bonobo_ui_component_add_listener (ui_component, "StatusBarOverwriteMode", 
 			(BonoboUIListenerFn)gedit_mdi_view_menu_item_toggled_handler, 
 			(gpointer)win);
-	
-	/* add a GeditRecentView object */
-	model = gedit_recent_get_model ();
-	view = EGG_RECENT_VIEW (egg_recent_view_bonobo_new (
-					ui_component, "/menu/File/Recents"));
-	egg_recent_view_bonobo_show_icons (EGG_RECENT_VIEW_BONOBO (view), FALSE);
-	egg_recent_view_set_model (view, model);
-	
-	g_signal_connect (G_OBJECT (view), "activate",
+
+
+	/* add a GeditRecent object */
+	recent = gedit_recent_new_with_ui_component ("gedit",
+						     gedit_settings->max_recents,
+						     ui_component,
+						     "/menu/File/Recents");
+	g_signal_connect (G_OBJECT (recent), "activate",
 			  G_CALLBACK (gedit_file_open_recent), NULL);
-	
-	g_object_set_data_full (G_OBJECT (win), RECENT_KEY, view,
-				g_object_unref);
+	g_object_set_data (G_OBJECT (win), RECENT_KEY, recent);
 
-		
-	/* Set window state and size, but only if the session is not being restored */
-	if (!bonobo_mdi_get_restoring_state (mdi))
-	{
-		state = gedit_prefs_manager_get_window_state ();
+	/* Set the window prefs. */
+	gtk_window_set_default_size (GTK_WINDOW (win), 
+			gedit_settings->window_width, 
+			gedit_settings->window_height);
 
-		if ((state & GDK_WINDOW_STATE_MAXIMIZED) != 0)
-		{
-			gtk_window_set_default_size (GTK_WINDOW (win),
-						     gedit_prefs_manager_get_default_window_width (),
-						     gedit_prefs_manager_get_default_window_height ());
-
-			gtk_window_maximize (GTK_WINDOW (win));
-		}
-		else
-		{
-			gtk_window_set_default_size (GTK_WINDOW (win), 
-						     gedit_prefs_manager_get_window_width (),
-						     gedit_prefs_manager_get_window_height ());
-
-			gtk_window_unmaximize (GTK_WINDOW (win));
-		}
-
-		if ((state & GDK_WINDOW_STATE_STICKY ) != 0)
-			gtk_window_stick (GTK_WINDOW (win));
-		else
-			gtk_window_unstick (GTK_WINDOW (win));
-	}
+	g_signal_connect (G_OBJECT (win), "configure_event",
+	                  G_CALLBACK (gedit_prefs_configure_event_handler), 
+			  NULL);
 	
 	/* Add the plugins menus */
 	gedit_plugins_engine_update_plugins_ui (win, TRUE);
 }
 
-static void
-gedit_mdi_app_destroy_handler (BonoboMDI *mdi, BonoboWindow *window)
-{
-	gedit_debug (DEBUG_MDI, "");
-	
-	g_return_if_fail (window != NULL);
-	g_return_if_fail (BONOBO_IS_WINDOW (window));
-
-	gedit_prefs_manager_save_window_size_and_state (window);
-}
-
 static void
 gedit_mdi_view_menu_item_toggled_handler (
 			BonoboUIComponent           *ui_component,
@@ -555,126 +345,76 @@ gedit_mdi_view_menu_item_toggled_handler (
 			BonoboWindow                *win)
 {
 	gboolean s;
-	GeditWindowPrefs *prefs;
 
 	gedit_debug (DEBUG_MDI, "%s toggled to '%s'", path, state);
 
-	prefs = gedit_window_prefs_get_from_window (win);
-	g_return_if_fail (prefs != NULL);
-
 	s = (strcmp (state, "1") == 0);
 
-	if (strcmp (path, "ViewToolbar") == 0)
+	if ((strcmp (path, "ViewToolbar") == 0) &&
+	    (s != gedit_settings->toolbar_visible))
 	{
-		if (s != prefs->toolbar_visible)
-		{
-			prefs->toolbar_visible = s;
-			gedit_mdi_set_app_toolbar_style (win);
-		}
-
-		goto save_prefs;
-	}
-
-	if (strcmp (path, "ViewStatusbar") == 0)
-	{
-		if (s != prefs->statusbar_visible)
-		{
-			prefs->statusbar_visible = s;
-			gedit_mdi_set_app_statusbar_style (win);
-		}
-
-		goto save_prefs;
-	}
-
-	if (strcmp (path, "ViewOutputWindow") == 0)
-	{
-		if (s != prefs->output_window_visible)
-		{
-			GtkWidget *ow;
-
-			prefs->output_window_visible = s;
-			gedit_mdi_set_app_statusbar_style (win);
-
-			ow = gedit_mdi_get_output_window_from_window (win);
-
-			if (prefs->output_window_visible)
-				gtk_widget_show (ow);
-			else
-				gtk_widget_hide (ow);
-		}	
+		gedit_settings->toolbar_visible = s;
+		gedit_mdi_set_app_toolbar_style (win);
 
 		return;
 	}
 
-
-	if (s && (strcmp (path, "ToolbarSystem") == 0))
+	if ((strcmp (path, "ViewStatusbar") == 0) &&
+	    (s != gedit_settings->statusbar_visible))
 	{
-		if (prefs->toolbar_buttons_style  != GEDIT_TOOLBAR_SYSTEM)
-		{
-			prefs->toolbar_buttons_style  = GEDIT_TOOLBAR_SYSTEM;
-			gedit_mdi_set_app_toolbar_style (win);
-		}
-
-		goto save_prefs;
+		gedit_settings->statusbar_visible = s;
+		gedit_mdi_set_app_statusbar_style (win);
 	}
 
-	if (s && (strcmp (path, "ToolbarIcon") == 0))
-	{
-		if (prefs->toolbar_buttons_style != GEDIT_TOOLBAR_ICONS)
-		{
-			prefs->toolbar_buttons_style = GEDIT_TOOLBAR_ICONS;
-			gedit_mdi_set_app_toolbar_style (win);
-		}
-		
-		goto save_prefs;
+	if (s && (strcmp (path, "ToolbarSystem") == 0) &&
+	    (gedit_settings->toolbar_buttons_style != GEDIT_TOOLBAR_SYSTEM))
+	{		
+		gedit_settings->toolbar_buttons_style = GEDIT_TOOLBAR_SYSTEM;
+		gedit_mdi_set_app_toolbar_style (win);
+
+		return;
 	}
 
-	if (s && (strcmp (path, "ToolbarIconText") == 0))
-	{
-		if (prefs->toolbar_buttons_style != GEDIT_TOOLBAR_ICONS_AND_TEXT)
-		{
-			prefs->toolbar_buttons_style = GEDIT_TOOLBAR_ICONS_AND_TEXT;
-			gedit_mdi_set_app_toolbar_style (win);
-		}
+	if (s && (strcmp (path, "ToolbarIcon") == 0) &&
+	    (gedit_settings->toolbar_buttons_style != GEDIT_TOOLBAR_ICONS))
+	{		
+		gedit_settings->toolbar_buttons_style = GEDIT_TOOLBAR_ICONS;
+		gedit_mdi_set_app_toolbar_style (win);
 
-		goto save_prefs;
+		return;
 	}
 
-	if (s && (strcmp (path, "ToolbarIconBothHoriz") == 0))
-	{
-		if (prefs->toolbar_buttons_style != GEDIT_TOOLBAR_ICONS_BOTH_HORIZ)
-		{
-			prefs->toolbar_buttons_style = GEDIT_TOOLBAR_ICONS_BOTH_HORIZ;
-			gedit_mdi_set_app_toolbar_style (win);
-		}
+	if (s && (strcmp (path, "ToolbarIconText") == 0) &&
+	    (gedit_settings->toolbar_buttons_style != GEDIT_TOOLBAR_ICONS_AND_TEXT))
+	{		
+		gedit_settings->toolbar_buttons_style = GEDIT_TOOLBAR_ICONS_AND_TEXT;
+		gedit_mdi_set_app_toolbar_style (win);
 
-		goto save_prefs;
+		return;
 	}
 
-	if (strcmp (path, "StatusBarCursorPosition") == 0)
+	if ((strcmp (path, "ToolbarTooltips") == 0) &&
+	    (s != gedit_settings->toolbar_view_tooltips))
 	{
-		if (s != prefs->statusbar_show_cursor_position)
-		{
-			prefs->statusbar_show_cursor_position = s;
-			gedit_mdi_set_app_statusbar_style (win);
-		}
+		gedit_settings->toolbar_view_tooltips = s;
+		gedit_mdi_set_app_toolbar_style (win);
 
-		goto save_prefs;
+		return;
 	}
 
-	if (strcmp (path, "StatusBarOverwriteMode") == 0)
+	if ((strcmp (path, "StatusBarCursorPosition") == 0) &&
+	    (s != gedit_settings->statusbar_view_cursor_position))
 	{
-		if (s != prefs->statusbar_show_overwrite_mode)
-		{
-			prefs->statusbar_show_overwrite_mode = s;
-			gedit_mdi_set_app_statusbar_style (win);
-		}
-
-		goto save_prefs;
+		gedit_settings->statusbar_view_cursor_position = s;
+		gedit_mdi_set_app_statusbar_style (win);
 	}
 
-save_prefs:
-	gedit_window_prefs_save (prefs);
+	if ((strcmp (path, "StatusBarOverwriteMode") == 0) &&
+	    (s != gedit_settings->statusbar_view_overwrite_mode))
+	{
+		gedit_settings->statusbar_view_overwrite_mode = s;
+		gedit_mdi_set_app_statusbar_style (win);
+	}
 }
 
 static void 
@@ -688,9 +428,6 @@ gedit_mdi_drag_data_received_handler (GtkWidget *widget, GdkDragContext *context
 	
 	gedit_debug (DEBUG_MDI, "");
 
-	if (info != TARGET_URI_LIST)
-		return;
-			
 	list = gnome_vfs_uri_list_parse (selection_data->data);
 	p = list;
 
@@ -704,7 +441,7 @@ gedit_mdi_drag_data_received_handler (GtkWidget *widget, GdkDragContext *context
 	
 	gnome_vfs_uri_list_free (list);
 
-	gedit_file_open_uri_list (file_list, 0, FALSE);	
+	gedit_file_open_uri_list (file_list, 0);	
 	
 	if (file_list == NULL)
 		return;
@@ -720,15 +457,13 @@ static void
 gedit_mdi_set_app_toolbar_style (BonoboWindow *win)
 {
 	BonoboUIComponent *ui_component;
-	GeditWindowPrefs *prefs = NULL;
+	GConfClient *client;
+	gchar *style = NULL;
 
 	gedit_debug (DEBUG_MDI, "");
 	
 	g_return_if_fail (BONOBO_IS_WINDOW (win));
-	
-	prefs = gedit_window_prefs_get_from_window (win);
-	g_return_if_fail (prefs != NULL);
-	
+			
 	ui_component = bonobo_mdi_get_ui_component_from_window (win);
 	g_return_if_fail (ui_component != NULL);
 			
@@ -737,46 +472,64 @@ gedit_mdi_set_app_toolbar_style (BonoboWindow *win)
 	/* Updated view menu */
 	gedit_menus_set_verb_state (ui_component, 
 				    "/commands/ViewToolbar",
-				    prefs->toolbar_visible);
+				    gedit_settings->toolbar_visible);
 
 	gedit_menus_set_verb_sensitive (ui_component, 
 				        "/commands/ToolbarSystem",
-				        prefs->toolbar_visible);
+				        gedit_settings->toolbar_visible);
 	gedit_menus_set_verb_sensitive (ui_component, 
 				        "/commands/ToolbarIcon",
-				        prefs->toolbar_visible);
+				        gedit_settings->toolbar_visible);
 	gedit_menus_set_verb_sensitive (ui_component, 
 				        "/commands/ToolbarIconText",
-				        prefs->toolbar_visible);
-	gedit_menus_set_verb_sensitive (ui_component, 
-				        "/commands/ToolbarIconBothHoriz",
-				        prefs->toolbar_visible);
+				        gedit_settings->toolbar_visible);
 	gedit_menus_set_verb_sensitive (ui_component, 
 				        "/commands/ToolbarTooltips",
-				        prefs->toolbar_visible);
+				        gedit_settings->toolbar_visible);
 
 	gedit_menus_set_verb_state (ui_component, 
 				    "/commands/ToolbarSystem",
-				    prefs->toolbar_buttons_style == GEDIT_TOOLBAR_SYSTEM);
+				    gedit_settings->toolbar_buttons_style == GEDIT_TOOLBAR_SYSTEM);
 
 	gedit_menus_set_verb_state (ui_component, 
 				    "/commands/ToolbarIcon",
-				    prefs->toolbar_buttons_style == GEDIT_TOOLBAR_ICONS);
+				    gedit_settings->toolbar_buttons_style == GEDIT_TOOLBAR_ICONS);
 
 	gedit_menus_set_verb_state (ui_component, 
 				    "/commands/ToolbarIconText",
-				    prefs->toolbar_buttons_style == GEDIT_TOOLBAR_ICONS_AND_TEXT);
+				    gedit_settings->toolbar_buttons_style == GEDIT_TOOLBAR_ICONS_AND_TEXT);
 
 	gedit_menus_set_verb_state (ui_component, 
-				    "/commands/ToolbarIconBothHoriz",
-				    prefs->toolbar_buttons_style == GEDIT_TOOLBAR_ICONS_BOTH_HORIZ);
+				    "/commands/ToolbarTooltips",
+				    gedit_settings->toolbar_view_tooltips);
+
 	
-	switch (prefs->toolbar_buttons_style)
+	/* Actually update toolbar style */
+	bonobo_ui_component_set_prop (
+		ui_component, "/Toolbar",
+		"tips", gedit_settings->toolbar_view_tooltips ? "1" : "0",
+		NULL);
+	
+	switch (gedit_settings->toolbar_buttons_style)
 	{
 		case GEDIT_TOOLBAR_SYSTEM:
-			gedit_debug (DEBUG_MDI, "GEDIT: SYSTEM");
-			bonobo_ui_component_set_prop (
-				ui_component, "/Toolbar", "look", "system", NULL);
+						
+			client = gconf_client_get_default ();
+			if (client == NULL) 
+				goto error;
+
+			style = gconf_client_get_string (client, 
+					"/desktop/gnome/interface/toolbar_style", NULL);
+
+			g_object_unref (G_OBJECT (client));
+			
+			if (style != NULL)
+			{
+				bonobo_ui_component_set_prop (
+					ui_component, "/Toolbar", "look", style, NULL);
+			
+				g_free (style);
+			}
 
 			break;
 			
@@ -793,13 +546,6 @@ gedit_mdi_set_app_toolbar_style (BonoboWindow *win)
 				ui_component, "/Toolbar", "look", "both", NULL);
 			
 			break;
-			
-		case GEDIT_TOOLBAR_ICONS_BOTH_HORIZ:
-			gedit_debug (DEBUG_MDI, "GEDIT: ICONS_BOTH_HORIZ");
-			bonobo_ui_component_set_prop (
-				ui_component, "/Toolbar", "look", "both_horiz", NULL);
-			
-			break;       
 		default:
 			goto error;
 			break;
@@ -807,7 +553,7 @@ gedit_mdi_set_app_toolbar_style (BonoboWindow *win)
 	
 	bonobo_ui_component_set_prop (
 			ui_component, "/Toolbar",
-			"hidden", prefs->toolbar_visible ? "0":"1", NULL);
+			"hidden", gedit_settings->toolbar_visible ? "0":"1", NULL);
 
  error:
 	bonobo_ui_component_thaw (ui_component, NULL);
@@ -816,54 +562,51 @@ gedit_mdi_set_app_toolbar_style (BonoboWindow *win)
 static void
 gedit_mdi_set_app_statusbar_style (BonoboWindow *win)
 {
-	GeditWindowPrefs *prefs = NULL;
 	BonoboUIComponent *ui_component;
 	GtkWidget *cp, *om;
 	
 	gedit_debug (DEBUG_MDI, "");
 	
 	g_return_if_fail (BONOBO_IS_WINDOW (win));
-
-	prefs = gedit_window_prefs_get_from_window (win);
-	g_return_if_fail (prefs != NULL);
-
+			
 	ui_component = bonobo_mdi_get_ui_component_from_window (win);
 	g_return_if_fail (ui_component != NULL);
 
 	bonobo_ui_component_freeze (ui_component, NULL);
-	
+
 	/* Update menu */
 	gedit_menus_set_verb_state (ui_component, 
 				    "/commands/ViewStatusbar",
-				    prefs->statusbar_visible);
+				    gedit_settings->statusbar_visible);
 
 	gedit_menus_set_verb_sensitive (ui_component, 
 				    "/commands/StatusBarCursorPosition",
-				    prefs->statusbar_visible);
+				    gedit_settings->statusbar_visible);
 
 	gedit_menus_set_verb_sensitive (ui_component, 
 				    "/commands/StatusBarOverwriteMode",
-				    prefs->statusbar_visible);
+				    gedit_settings->statusbar_visible);
 
 	gedit_menus_set_verb_state (ui_component, 
 				    "/commands/StatusBarCursorPosition",
-				    prefs->statusbar_show_cursor_position);
+				    gedit_settings->statusbar_view_cursor_position);
 
 	gedit_menus_set_verb_state (ui_component, 
 				    "/commands/StatusBarOverwriteMode",
-				    prefs->statusbar_show_overwrite_mode);
+				    gedit_settings->statusbar_view_overwrite_mode);
+
 	
 	/* Actually update status bar style */
 	bonobo_ui_component_set_prop (
 		ui_component, "/status",
-		"hidden", prefs->statusbar_visible ? "0" : "1",
+		"hidden", gedit_settings->statusbar_visible ? "0" : "1",
 		NULL);
 
 	cp = GTK_WIDGET (g_object_get_data (G_OBJECT (win), "CursorPosition"));
 	if (cp == NULL)
 		goto error;
 
-	if (prefs->statusbar_show_cursor_position)
+	if (gedit_settings->statusbar_view_cursor_position)
 	{
 		bonobo_ui_component_set_prop (
 			ui_component, "/status/CursorPosition", "hidden", "0", NULL);
@@ -882,7 +625,7 @@ gedit_mdi_set_app_statusbar_style (BonoboWindow *win)
 	if (om == NULL)
 		goto error;
 
-	if (prefs->statusbar_show_overwrite_mode)
+	if (gedit_settings->statusbar_view_overwrite_mode)
 	{
 		gtk_widget_show (om);
 		
@@ -897,13 +640,13 @@ gedit_mdi_set_app_statusbar_style (BonoboWindow *win)
 		gtk_widget_hide (om);
 	}
 
-	if (!prefs->statusbar_show_overwrite_mode)
+	if (!gedit_settings->statusbar_view_overwrite_mode)
 		gtk_statusbar_set_has_resize_grip (GTK_STATUSBAR (cp), TRUE);
 	else
 		gtk_statusbar_set_has_resize_grip (GTK_STATUSBAR (cp), FALSE);
 
-	if (!prefs->statusbar_show_cursor_position &&
-	    !prefs->statusbar_show_overwrite_mode)
+	if (!gedit_settings->statusbar_view_cursor_position &&
+	    !gedit_settings->statusbar_view_overwrite_mode)
 		bonobo_ui_component_set_prop (
 			ui_component, "/status", "resize_grip", "1", NULL);
 	else
@@ -956,23 +699,25 @@ gedit_mdi_add_child_handler (BonoboMDI *mdi, BonoboMDIChild *child)
 static gint 
 gedit_mdi_add_view_handler (BonoboMDI *mdi, GtkWidget *view)
 {
-	GtkTextView *text_view;
-	GtkTargetList *tl;
+	static GtkTargetEntry drag_types[] =
+	{
+		{ "text/uri-list", 0, 0 },
+	};
+
+	static gint n_drag_types = sizeof (drag_types) / sizeof (drag_types [0]);
 
 	gedit_debug (DEBUG_MDI, "");
-
-	g_return_val_if_fail (view != NULL, TRUE);
-
-	text_view = gedit_view_get_gtk_text_view (GEDIT_VIEW (view));
-	g_return_val_if_fail (text_view != NULL, TRUE);
 	
+	/* FIXME */
 	/* Drag and drop support */
-	tl = gtk_drag_dest_get_target_list (GTK_WIDGET (text_view));
-	g_return_val_if_fail (tl != NULL, TRUE);
-
-	gtk_target_list_add_table (tl, drag_types, n_drag_types);
-
-	g_signal_connect (G_OBJECT (text_view), "drag_data_received",
+	gtk_drag_dest_set (view,
+			   GTK_DEST_DEFAULT_MOTION |
+			   GTK_DEST_DEFAULT_HIGHLIGHT |
+			   GTK_DEST_DEFAULT_DROP,
+			   drag_types, n_drag_types,
+			   GDK_ACTION_COPY);
+		
+	g_signal_connect (G_OBJECT (view), "drag_data_received",
 			  G_CALLBACK (gedit_mdi_drag_data_received_handler), 
 			  NULL);
 
@@ -984,8 +729,6 @@ gedit_mdi_remove_child_handler (BonoboMDI *mdi, BonoboMDIChild *child)
 {
 	GeditDocument* doc;
 	gboolean close = TRUE;
-	gchar *raw_uri;
-	gboolean deleted = FALSE;
 	
 	gedit_debug (DEBUG_MDI, "");
 
@@ -994,17 +737,7 @@ gedit_mdi_remove_child_handler (BonoboMDI *mdi, BonoboMDIChild *child)
 
 	doc = GEDIT_MDI_CHILD (child)->document;
 
-	raw_uri = gedit_document_get_raw_uri (doc); 
-	if (raw_uri != NULL)
-	{
-		if (gedit_document_is_readonly (doc))
-			deleted = FALSE;
-		else
-			deleted = !gedit_utils_uri_exists (raw_uri);
-	}
-	g_free (raw_uri);
-				
-	if (gedit_document_get_modified (doc) || deleted)
+	if (gedit_document_get_modified (doc))
 	{
 		GtkWidget *msgbox, *w;
 		gchar *fname = NULL, *msg = NULL;
@@ -1014,14 +747,7 @@ gedit_mdi_remove_child_handler (BonoboMDI *mdi, BonoboMDIChild *child)
 		w = GTK_WIDGET (g_list_nth_data (bonobo_mdi_child_get_views (child), 0));
 			
 		if(w != NULL)
-		{
-			GtkWindow *window;
-
-			window = GTK_WINDOW (bonobo_mdi_get_window_from_view (w));
-			gtk_window_present (window);
-			
 			bonobo_mdi_set_active_view (mdi, w);
-		}
 
 		fname = gedit_document_get_short_name (doc);
 
@@ -1071,9 +797,6 @@ gedit_mdi_remove_child_handler (BonoboMDI *mdi, BonoboMDIChild *child)
 
 		gtk_window_set_resizable (GTK_WINDOW (msgbox), FALSE);
 
-		gtk_widget_show (msgbox);
-		gtk_window_present (GTK_WINDOW (msgbox));
-
 		ret = gtk_dialog_run (GTK_DIALOG (msgbox));
 		
 		gtk_widget_destroy (msgbox);
@@ -1084,7 +807,7 @@ gedit_mdi_remove_child_handler (BonoboMDI *mdi, BonoboMDIChild *child)
 		switch (ret)
 		{
 			case GTK_RESPONSE_YES:
-				close = gedit_file_save (GEDIT_MDI_CHILD (child), TRUE);
+				close = gedit_file_save (GEDIT_MDI_CHILD (child));
 				break;
 			case GTK_RESPONSE_NO:
 				close = TRUE;
@@ -1201,9 +924,7 @@ void gedit_mdi_view_changed_handler (BonoboMDI *mdi, GtkWidget *old_view)
 	if (active_view == NULL)
 		return;
 
-	/*
 	gtk_widget_grab_focus (active_view);
-	*/
 
 	status = g_object_get_data (G_OBJECT (win), "CursorPosition");	
 	gedit_view_set_cursor_position_statusbar (GEDIT_VIEW (active_view), status);
@@ -1262,7 +983,7 @@ gedit_mdi_set_active_window_verbs_sensitivity (BonoboMDI *mdi)
 	bonobo_ui_component_freeze (ui_component, NULL);
 	
 	gedit_plugins_engine_update_plugins_ui (active_window, FALSE);
-	
+
 	if (active_child == NULL)
 	{
 		gedit_menus_set_verb_list_sensitive (ui_component, 
@@ -1275,9 +996,6 @@ gedit_mdi_set_active_window_verbs_sensitivity (BonoboMDI *mdi)
 				gedit_menus_all_sensible_verbs, TRUE);
 	}
 
-	gedit_menus_set_verb_sensitive (ui_component, "/commands/DocumentsMoveToNewWindow",
-				(bonobo_mdi_n_children_for_window (active_window) > 1) ? TRUE : FALSE);
-
 	doc = GEDIT_MDI_CHILD (active_child)->document;
 	g_return_if_fail (doc != NULL);
 	
@@ -1343,189 +1061,117 @@ gedit_mdi_set_active_window_undo_redo_verbs_sensitivity (BonoboMDI *mdi)
 	bonobo_ui_component_thaw (ui_component, NULL);
 }
 
-EggRecentView *
-gedit_mdi_get_recent_view_from_window (BonoboWindow *win)
+void
+gedit_mdi_update_ui_according_to_preferences (GeditMDI *mdi)
+{
+#if 0
+	GList *windows;		
+#endif
+	GList *children;
+	GdkColor background;
+	GdkColor text;
+	GdkColor selection;
+	GdkColor sel_text;
+	gchar* font;
+
+	gedit_debug (DEBUG_MDI, "");
+
+#if 0
+	windows = bonobo_mdi_get_windows (BONOBO_MDI (mdi));
+
+	while (windows != NULL)
+	{
+		BonoboUIComponent *ui_component;
+		BonoboWindow *active_window = BONOBO_WINDOW (windows->data);
+		g_return_if_fail (active_window != NULL);
+		
+		ui_component = bonobo_mdi_get_ui_component_from_window (active_window);
+		g_return_if_fail (ui_component != NULL);
+
+		bonobo_ui_component_freeze (ui_component, NULL);
+
+		gedit_mdi_set_app_statusbar_style (active_window);
+		gedit_mdi_set_app_toolbar_style (active_window);
+
+		bonobo_ui_component_thaw (ui_component, NULL);
+
+		windows = windows->next;
+	}
+#endif
+	children = bonobo_mdi_get_children (BONOBO_MDI (mdi));
+
+	if (gedit_settings->use_default_colors)
+	{
+		GtkStyle *style;
+	       
+		style = gtk_style_new ();
+
+		background = style->base [GTK_STATE_NORMAL];
+		text = style->text [GTK_STATE_NORMAL];
+		sel_text = style->text [GTK_STATE_SELECTED];
+		selection = style->base [GTK_STATE_SELECTED];
+
+		g_object_unref (G_OBJECT (style));
+	}
+	else
+	{
+		background = gedit_settings->background_color;
+		text = gedit_settings->text_color;
+		selection = gedit_settings->selection_color;
+		sel_text = gedit_settings->selected_text_color;
+	}
+
+	if (gedit_settings->use_default_font)
+	{
+		GtkStyle *style;
+		
+		style = gtk_style_new ();
+
+		font = pango_font_description_to_string (style->font_desc);
+		
+		if (font == NULL)
+			/* Fallback */
+			font = g_strdup (gedit_settings->editor_font);
+
+		g_object_unref (G_OBJECT (style));
+
+	}
+	else
+		font = g_strdup (gedit_settings->editor_font);
+
+	while (children != NULL)
+	{
+		GList *views = bonobo_mdi_child_get_views (BONOBO_MDI_CHILD (children->data));
+
+		while (views != NULL)
+		{
+			GeditView *v =	GEDIT_VIEW (views->data);
+			
+			gedit_view_set_colors (v, &background, &text, &selection, &sel_text);
+			gedit_view_set_font (v, font);
+			gedit_view_set_wrap_mode (v, gedit_settings->wrap_mode);
+			gedit_view_show_line_numbers (v, gedit_settings->show_line_numbers);
+			gedit_view_set_tab_size (v, gedit_settings->tab_size);
+			views = views->next;
+		}
+		
+		children = children->next;
+	}
+
+	g_free (font);
+
+/*
+	bonobo_mdi_set_mode (BONOBO_MDI (mdi), gedit_settings->mdi_mode);
+*/
+}
+
+GeditRecent*
+gedit_mdi_get_recent_from_window (BonoboWindow* win)
 {
 	gpointer r;
 	gedit_debug (DEBUG_MDI, "");
 
 	r = g_object_get_data (G_OBJECT (win), RECENT_KEY);
 	
-	return (r != NULL) ? EGG_RECENT_VIEW (r) : NULL;
-}
-
-static void
-gedit_mdi_close_output_window_cb (GtkWidget *widget, gpointer user_data)
-{
-	BonoboWindow *bw;
-	
-	bw = BONOBO_WINDOW (user_data);
-
-	gtk_widget_hide (widget);
-}
-
-static void
-gedit_mdi_show_output_window_cb (GtkWidget *widget, gpointer user_data)
-{
-	BonoboWindow *bw;
-	GeditWindowPrefs *prefs;
-	BonoboUIComponent *ui_component;
-	
-	gedit_debug (DEBUG_MDI, "");
-	
-	bw = BONOBO_WINDOW (user_data);
-	g_return_if_fail (bw != NULL);
-	
-	ui_component = bonobo_mdi_get_ui_component_from_window (bw);
-	g_return_if_fail (ui_component != NULL);
-	
-	prefs = gedit_window_prefs_get_from_window (bw);
-	g_return_if_fail (prefs != NULL);
-	
-	prefs->output_window_visible = TRUE;
-	
-	gedit_menus_set_verb_state (ui_component, 
-				    "/commands/ViewOutputWindow",
-				    prefs->output_window_visible);
-}
-
-static void
-gedit_mdi_hide_output_window_cb (GtkWidget *widget, gpointer user_data)
-{
-	BonoboWindow *bw;
-	GeditWindowPrefs *prefs;
-	BonoboUIComponent *ui_component;
-	
-	gedit_debug (DEBUG_MDI, "");
-	
-	bw = BONOBO_WINDOW (user_data);
-	g_return_if_fail (bw != NULL);
-	
-	ui_component = bonobo_mdi_get_ui_component_from_window (bw);
-	g_return_if_fail (ui_component != NULL);
-
-	prefs = gedit_window_prefs_get_from_window (bw);
-	g_return_if_fail (prefs != NULL);
-
-	prefs->output_window_visible = FALSE;
-
-	gedit_menus_set_verb_state (ui_component, 
-				    "/commands/ViewOutputWindow",
-				    prefs->output_window_visible);
-}
-
-
-GtkWidget *
-gedit_mdi_get_output_window_from_window (BonoboWindow *win)
-{
-	gpointer r;
-	gedit_debug (DEBUG_MDI, "");
-
-	r = g_object_get_data (G_OBJECT (win), OUTPUT_WINDOW_KEY);
-
-	if (r == NULL)
-	{
-		GtkWidget *ow;
-
-		/* Add output window */
-		ow = gedit_output_window_new ();
-		bonobo_mdi_set_bottom_pane_for_window (BONOBO_WINDOW (win), ow);
-		gtk_widget_show_all (ow);
-		g_signal_connect (G_OBJECT (ow), "close_requested",
-			  G_CALLBACK (gedit_mdi_close_output_window_cb), win);
-		g_signal_connect (G_OBJECT (ow), "show",
-			  G_CALLBACK (gedit_mdi_show_output_window_cb), win);
-		g_signal_connect (G_OBJECT (ow), "hide",
-			  G_CALLBACK (gedit_mdi_hide_output_window_cb), win);
-		g_object_set_data (G_OBJECT (win), OUTPUT_WINDOW_KEY, ow);
-
-		return ow;
-	}
-	
-	return (r != NULL) ? GTK_WIDGET (r) : NULL;
-}
-
-
-
-static GeditWindowPrefs *
-gedit_window_prefs_new (void)
-{
-	GeditWindowPrefs *prefs;
-
-	gedit_debug (DEBUG_MDI, "");
-
-	prefs = g_new0 (GeditWindowPrefs, 1);
-
-	prefs->toolbar_visible = gedit_prefs_manager_get_toolbar_visible ();
-	prefs->toolbar_buttons_style = gedit_prefs_manager_get_toolbar_buttons_style ();
-
-	prefs->statusbar_visible = gedit_prefs_manager_get_statusbar_visible ();
-	prefs->statusbar_show_cursor_position = gedit_prefs_manager_get_statusbar_show_cursor_position ();
-	prefs->statusbar_show_overwrite_mode = gedit_prefs_manager_get_statusbar_show_overwrite_mode ();
-
-	prefs->output_window_visible = FALSE;
-
-	return prefs;
-}
-
-static void
-gedit_window_prefs_attach_to_window (GeditWindowPrefs *prefs, BonoboWindow *win)
-{
-	gedit_debug (DEBUG_MDI, "");
-
-	g_return_if_fail (prefs != NULL);
-	g_return_if_fail (win != NULL);
-	g_return_if_fail (BONOBO_IS_WINDOW (win));
-
-	if (!window_prefs_id)
-		window_prefs_id = g_quark_from_static_string ("GeditWindowPrefsData");
-
-	g_object_set_qdata_full (G_OBJECT (win), window_prefs_id, prefs, g_free);
-}
-
-static GeditWindowPrefs	*
-gedit_window_prefs_get_from_window (BonoboWindow *win)
-{
-	GeditWindowPrefs *prefs;
-
-	gedit_debug (DEBUG_MDI, "");
-
-	g_return_val_if_fail (win != NULL, NULL);
-	g_return_val_if_fail (BONOBO_IS_WINDOW (win), NULL);
-
-	prefs = g_object_get_qdata (G_OBJECT (win), window_prefs_id);
-
-	return (prefs != NULL) ? (GeditWindowPrefs*)prefs : NULL;
-}
-
-static void
-gedit_window_prefs_save (GeditWindowPrefs *prefs)
-{
-	gedit_debug (DEBUG_MDI, "");
-
-	g_return_if_fail (prefs != NULL);
-
-	if ((prefs->toolbar_visible != gedit_prefs_manager_get_toolbar_visible ()) &&
-	     gedit_prefs_manager_toolbar_visible_can_set ())
-		gedit_prefs_manager_set_toolbar_visible (prefs->toolbar_visible);
-
-	if ((prefs->toolbar_buttons_style != gedit_prefs_manager_get_toolbar_buttons_style ()) &&
-	    gedit_prefs_manager_toolbar_buttons_style_can_set ())
-		gedit_prefs_manager_set_toolbar_buttons_style (prefs->toolbar_buttons_style);
-
-	if ((prefs->statusbar_visible != gedit_prefs_manager_get_statusbar_visible ()) &&
-	    gedit_prefs_manager_get_statusbar_visible ())
-		gedit_prefs_manager_set_statusbar_visible (prefs->statusbar_visible);
-
-	if ((prefs->statusbar_show_cursor_position != 
-			gedit_prefs_manager_get_statusbar_show_cursor_position ()) &&
-	    gedit_prefs_manager_statusbar_show_cursor_position_can_set ())
-		gedit_prefs_manager_set_statusbar_show_cursor_position (
-				prefs->statusbar_show_cursor_position);
-
-	if ((prefs->statusbar_show_overwrite_mode != 
-			gedit_prefs_manager_get_statusbar_show_overwrite_mode ()) &&
-	    gedit_prefs_manager_statusbar_show_overwrite_mode_can_set ())
-		gedit_prefs_manager_set_statusbar_show_overwrite_mode (
-				prefs->statusbar_show_overwrite_mode);
+	return (r != NULL) ? GEDIT_RECENT (r) : NULL;
 }
